##############################################################################
#
# Copyright (c) 2017 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
Tests for zope.component.interfaces
"""

import unittest


class TestInterfaces(unittest.TestCase):

    def test_nothing(self):
        """
        This exists only so that zope.testrunner doesn't complain this
        module has no tests.

        Even though this file is empty, we want to keep it around for ease
        of browsing the history.
        """



##############################################################################
#
# Copyright (c) 2012 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Tests for z.c.registry
"""
import unittest

from zope.component.tests import fails_if_called


class Test_dispatchUtilityRegistrationEvent(unittest.TestCase):

    from zope.component.testing import setUp
    from zope.component.testing import tearDown

    def _callFUT(self, *args, **kw):
        from zope.component.registry import dispatchUtilityRegistrationEvent
        return dispatchUtilityRegistrationEvent(*args, **kw)

    def test_it(self):
        from zope.component import registry

        class _Registration:
            component = object()
        _EVENT = object()
        _handled = []

        def _handle(*args):
            _handled.append(args)
        with _Monkey(registry, handle=_handle):
            self._callFUT(_Registration(), _EVENT)
        self.assertEqual(_handled, [(_Registration.component, _EVENT)])


class Test_dispatchAdapterRegistrationEvent(unittest.TestCase):

    from zope.component.testing import setUp
    from zope.component.testing import tearDown

    def _callFUT(self, *args, **kw):
        from zope.component.registry import dispatchAdapterRegistrationEvent
        return dispatchAdapterRegistrationEvent(*args, **kw)

    def test_it(self):
        from zope.component import registry

        class _Registration:
            factory = fails_if_called(self)
        _registration = _Registration()
        _EVENT = object()
        _handled = []

        def _handle(*args):
            _handled.append(args)
        with _Monkey(registry, handle=_handle):
            self._callFUT(_registration, _EVENT)
        self.assertEqual(_handled, [(_registration.factory, _EVENT)])


class Test_dispatchSubscriptionAdapterRegistrationEvent(unittest.TestCase):

    from zope.component.testing import setUp
    from zope.component.testing import tearDown

    def _callFUT(self, *args, **kw):
        from zope.component.registry import \
            dispatchSubscriptionAdapterRegistrationEvent
        return dispatchSubscriptionAdapterRegistrationEvent(*args, **kw)

    def test_it(self):
        from zope.component import registry

        class _Registration:
            factory = fails_if_called(self)
        _registration = _Registration()
        _EVENT = object()
        _handled = []

        def _handle(*args):
            _handled.append(args)
        with _Monkey(registry, handle=_handle):
            self._callFUT(_registration, _EVENT)
        self.assertEqual(_handled, [(_registration.factory, _EVENT)])


class Test_dispatchHandlerRegistrationEvent(unittest.TestCase):

    from zope.component.testing import setUp
    from zope.component.testing import tearDown

    def _callFUT(self, *args, **kw):
        from zope.component.registry import dispatchHandlerRegistrationEvent
        return dispatchHandlerRegistrationEvent(*args, **kw)

    def test_it(self):
        from zope.component import registry

        class _Registration:
            handler = fails_if_called(self)
        _registration = _Registration()
        _EVENT = object()
        _handled = []

        def _handle(*args):
            _handled.append(args)
        with _Monkey(registry, handle=_handle):
            self._callFUT(_registration, _EVENT)
        self.assertEqual(_handled, [(_registration.handler, _EVENT)])


class _Monkey:
    # context-manager for replacing module names in the scope of a test.
    def __init__(self, module, **kw):
        self.module = module
        self.to_restore = {key: getattr(module, key) for key in kw}
        for key, value in kw.items():
            setattr(module, key, value)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        for key, value in self.to_restore.items():
            setattr(self.module, key, value)



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Adapter declarations
"""
import sys


class adapter:
    """
    Decorator that declares that the decorated object adapts the given
    *interfaces*.

    This is commonly used in conjunction with :obj:`zope.interface.implementer`
    to declare what adapting the *interfaces* will provide.
    """

    def __init__(self, *interfaces):
        self.interfaces = interfaces

    def __call__(self, ob):
        if isinstance(ob, type):
            ob.__component_adapts__ = _adapts_descr(self.interfaces)
        else:
            ob.__component_adapts__ = self.interfaces

        return ob


def adapts(*interfaces):
    frame = sys._getframe(1)
    locals = frame.f_locals

    # Ensure we were called from a class def.
    if locals is frame.f_globals or '__module__' not in locals:
        raise TypeError("adapts can be used only from a class definition.")

    if '__component_adapts__' in locals:
        raise TypeError("adapts can be used only once in a class definition.")

    locals['__component_adapts__'] = _adapts_descr(interfaces)


def adaptedBy(ob):
    """
    Return the *interfaces* that *ob* will adapt, as declared by
    :obj:`adapter`.
    """
    return getattr(ob, '__component_adapts__', None)


def getName(ob):
    return getattr(ob, '__component_name__', '')


class _adapts_descr:
    def __init__(self, interfaces):
        self.interfaces = interfaces

    def __get__(self, inst, cls):
        if inst is None:
            return self.interfaces
        raise AttributeError('__component_adapts__')



############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
############################################################################
"""
Component and Component Architecture Interfaces

The `IComponentArchitecture` and `IComponentRegistrationConvenience` interfaces
are provided by `zope.component` directly.
"""
from zope.interface import Attribute
from zope.interface import Interface


# pylint:disable=inherit-non-class,no-self-argument,no-method-argument

class IComponentArchitecture(Interface):
    """The Component Architecture is defined by two key components: Adapters
    and Utiltities. Both are managed by site managers. All other components
    build on top of them.
    """
    # Site Manager API

    def getGlobalSiteManager():
        """Return the global site manager.

        This function should never fail and always return an object that
        provides `zope.interface.interfaces.IComponents`.
        """

    def getSiteManager(context=None):
        """Get the nearest site manager in the given context.

        If *context* is `None`, return the global site manager.

        If the *context* is not `None`, it is expected that an adapter
        from the *context* to
        :mod:`zope.interface.interfaces.IComponentLookup` can be
        found. If no adapter is found, a
        `~zope.interface.interfaces.ComponentLookupError` is raised.
        """

    # Utility API

    def getUtility(interface, name='', context=None):
        """Get the utility that provides interface

        Returns the nearest utility to the context that implements the
        specified interface. If one is not found, raises
        `~zope.interface.interfaces.ComponentLookupError`.
        """

    def queryUtility(interface, name='', default=None, context=None):
        """Look for the utility that provides *interface*

        Returns the nearest utility to the *context* that implements the
        specified interface. If one is not found, returns *default*.
        """

    def queryNextUtility(context, interface, name='', default=None):
        """Query for the next available utility.

        Find the next available utility providing *interface* and having the
        specified name. If no utility was found, return the specified *default*
        value.
        """

    def getNextUtility(context, interface, name=''):
        """Get the next available utility.

        If no utility was found, a
        `~zope.interface.interfaces.ComponentLookupError` is raised.
        """

    def getUtilitiesFor(interface, context=None):
        """Return the utilities that provide an interface

        An iterable of utility name-value pairs is returned.
        """

    def getAllUtilitiesRegisteredFor(interface, context=None):
        """Return all registered utilities for an interface

        This includes overridden utilities.

        An iterable of utility instances is returned.  No names are
        returned.
        """

    # Adapter API

    def getAdapter(object,
                   interface=Interface, name='',
                   context=None):
        """Get a named adapter to an interface for an object

        Returns an adapter that can adapt object to interface.  If a matching
        adapter cannot be found, raises
        `~zope.interface.interfaces.ComponentLookupError`.
        """

    def getAdapterInContext(object, interface, context):
        """Get a special adapter to an interface for an object

        .. note::
            This method should only be used if a custom context
            needs to be provided to provide custom component
            lookup. Otherwise, call the interface, as in::

               interface(object)

        Returns an adapter that can adapt object to interface. If a
        matching adapter cannot be found, raises
        `~zope.interface.interfaces.ComponentLookupError`.

        If the object has a ``__conform__`` method, this method will be
        called with the requested interface. If the method returns a
        non-None value, that value will be returned. Otherwise, if the
        object already implements the interface, the object will be
        returned.
        """

    def getMultiAdapter(objects,
                        interface=Interface, name='',
                        context=None):
        """Look for a multi-adapter to an *interface* for an *objects*

        Returns a multi-adapter that can adapt *objects* to *interface*.
        If a matching adapter cannot be found, raises
        `~zope.interface.interfaces.ComponentLookupError`.

        The name consisting of an empty string is reserved for unnamed
        adapters. The unnamed adapter methods will often call the
        named adapter methods with an empty string for a name.
        """

    def queryAdapter(object, interface=Interface, name='',
                     default=None, context=None):
        """Look for a named adapter to an interface for an object

        Returns an adapter that can adapt object to interface. If a
        matching adapter cannot be found, returns the default.
        """

    def queryAdapterInContext(object, interface, context, default=None):
        """
        Look for a special adapter to an interface for an object

        .. note::

            This method should only be used if a custom context
            needs to be provided to provide custom component lookup.
            Otherwise, call the interface, as in::

                interface(object, default)

        Returns an adapter that can adapt object to interface. If a
        matching adapter cannot be found, returns the default.

        If the object has a ``__conform__`` method, this method will be
        called with the requested interface. If the method returns a
        non-None value, that value will be returned. Otherwise, if the
        object already implements the interface, the object will be
        returned.
        """

    def queryMultiAdapter(objects,
                          interface=Interface, name='',
                          default=None,
                          context=None):
        """Look for a multi-adapter to an interface for objects

        Returns a multi-adapter that can adapt objects to interface.  If a
        matching adapter cannot be found, returns the default.

        The name consisting of an empty string is reserved for unnamed
        adapters. The unnamed adapter methods will often call the
        named adapter methods with an empty string for a name.
        """

    def getAdapters(objects, provided, context=None):
        """
        Look for all matching adapters to a provided interface for
        objects

        Return a list of adapters that match. If an adapter is named,
        only the most specific adapter of a given name is returned.
        """

    def subscribers(required, provided, context=None):
        """Get subscribers

        Subscribers are returned that provide the provided interface
        and that depend on and are computed from the sequence of
        required objects.
        """

    def handle(*objects):
        """Call all of the handlers for the given objects

        Handlers are subscription adapter factories that don't produce
        anything. They do all of their work when called. Handlers are
        typically used to handle events.
        """

    def adapts(*interfaces):
        """
        Declare that a class adapts the given interfaces.

        This function can only be used in a class definition.

        (TODO, allow classes to be passed as well as interfaces.)
        """

    # Factory service

    def createObject(factory_name, *args, **kwargs):
        """Create an object using a factory

        Finds the named factory in the current site and calls it with
        the given arguments. If a matching factory cannot be found
        raises `~zope.interface.interfaces.ComponentLookupError`.
        Returns the created object.

        A context keyword argument can be provided to cause the
        factory to be looked up in a location other than the current
        site. (Of course, this means that it is impossible to pass a
        keyword argument named "context" to the factory.
        """

    def getFactoryInterfaces(name, context=None):
        """Get interfaces implemented by a factory

        Finds the factory of the given name that is nearest to the
        context, and returns the interface or interface tuple that
        object instances created by the named factory will implement.
        """

    def getFactoriesFor(interface, context=None):
        """Return a tuple (name, factory) of registered factories that
        create objects which implement the given interface.
        """


class IRegistry(Interface):
    """Object that supports component registry
    """

    def registrations():
        """Return an iterable of component registrations
        """


class IComponentRegistrationConvenience(Interface):
    """API for registering components.

    .. caution::
        This API should only be used from test or
        application-setup code. This api shouldn't be used by regular
        library modules, as component registration is a configuration
        activity.
    """

    def provideUtility(component, provides=None, name=''):
        """Register a utility globally

        A utility is registered to provide an interface with a
        name. If a component provides only one interface, then the
        provides argument can be omitted and the provided interface
        will be used. (In this case, provides argument can still be
        provided to provide a less specific interface.)
        """

    def provideAdapter(factory, adapts=None, provides=None, name=''):
        """Register an adapter globally

        An adapter is registered to provide an interface with a name
        for some number of object types. If a factory implements only
        one interface, then the provides argument can be omitted and
        the provided interface will be used. (In this case, a provides
        argument can still be provided to provide a less specific
        interface.)

        If the factory has an adapts declaration, then the adapts
        argument can be omitted and the declaration will be used.  (An
        adapts argument can be provided to override the declaration.)
        """

    def provideSubscriptionAdapter(factory, adapts=None, provides=None):
        """Register a subscription adapter

        A subscription adapter is registered to provide an interface
        for some number of object types. If a factory implements only
        one interface, then the provides argument can be omitted and
        the provided interface will be used. (In this case, a provides
        argument can still be provided to provide a less specific
        interface.)

        If the factory has an adapts declaration, then the adapts
        argument can be omitted and the declaration will be used.  (An
        adapts argument can be provided to override the declaration.)
        """

    def provideHandler(handler, adapts=None):
        """Register a handler

        Handlers are subscription adapter factories that don't produce
        anything.  They do all of their work when called.  Handlers
        are typically used to handle events.

        If the handler has an adapts declaration, then the adapts
        argument can be omitted and the declaration will be used.  (An
        adapts argument can be provided to override the declaration.)
        """


class IPossibleSite(Interface):
    """An object that could be a site.
    """

    def setSiteManager(sitemanager):
        """Sets the site manager for this object.
        """

    def getSiteManager():
        """Returns the site manager contained in this object.

        If there isn't a site manager, raise a component lookup.
        """


class ISite(IPossibleSite):
    """Marker interface to indicate that we have a site"""


class Misused(Exception):
    """A component is being used (registered) for the wrong interface."""


class IFactory(Interface):
    """A factory is responsible for creating other components."""

    title = Attribute("The factory title.")

    description = Attribute("A brief description of the factory.")

    def __call__(*args, **kw):
        """Return an instance of the objects we're a factory for."""

    def getInterfaces():
        """Get the interfaces implemented by the factory

        Return the interface(s), as an instance of Implements, that objects
        created by this factory will implement. If the callable's Implements
        instance cannot be created, an empty Implements instance is returned.
        """


# Internal helpers

def _inherits_docs(func, iface):
    doc = iface[func.__name__].__doc__

    # doc can be None if the interpreter drops all docstrings when the
    # environment variable PYTHONOPTIMIZE is set 2.
    if doc is None:
        return func  # pragma: no cover

    # By adding the ..seealso:: we get a link from our overview page
    # to the specific narrative place where the function is described, because
    # our overview page uses :noindex:
    doc += "\n        .. seealso::"
    doc += f"\n           Function `~zope.component.{func.__name__}` for"
    doc += " notes, and "
    doc += f"\n           `~zope.component.interfaces.{iface.__name__}` for"
    doc += " the defining interface.\n"
    func.__doc__ = doc
    return func


def inherits_arch_docs(func):
    return _inherits_docs(func, IComponentArchitecture)


def inherits_reg_docs(func):
    return _inherits_docs(func, IComponentRegistrationConvenience)



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Factory object
"""
from zope.interface import implementedBy
from zope.interface import implementer
from zope.interface.declarations import Implements

from zope.component.interfaces import IFactory


@implementer(IFactory)
class Factory:
    """Generic factory implementation.

    The purpose of this implementation is to provide a quick way of creating
    factories for classes, functions and other objects.
    """

    def __init__(self, callable, title='', description='', interfaces=None):
        self._callable = callable
        self.title = title
        self.description = description
        self._interfaces = interfaces

    def __call__(self, *args, **kw):
        return self._callable(*args, **kw)

    def getInterfaces(self):
        if self._interfaces is not None:
            spec = Implements(*self._interfaces)
            spec.__name__ = getattr(self._callable, '__name__', '[callable]')
            return spec
        return implementedBy(self._callable)

    def __repr__(self):  # pragma: no cover
        return f'<{self.__class__.__name__} for {self._callable!r}>'



##############################################################################
#
# Copyright (c) 2012 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Test adapter declaration helpers
"""
import unittest


class Test_dispatch(unittest.TestCase):

    def test_it(self):
        from zope.interface import Interface

        from zope.component.event import dispatch
        from zope.component.globalregistry import getGlobalSiteManager
        _adapted = []

        def _adapter(context):
            _adapted.append(context)
            return object()
        gsm = getGlobalSiteManager()
        gsm.registerHandler(_adapter, (Interface,))
        del _adapted[:]  # clear handler reg
        event = object()
        dispatch(event)
        self.assertEqual(_adapted, [event])


class Test_objectEventNotify(unittest.TestCase):

    def test_it(self):
        from zope.interface import Interface
        from zope.interface import implementer
        from zope.interface.interfaces import IObjectEvent

        from zope.component.event import objectEventNotify
        from zope.component.globalregistry import getGlobalSiteManager
        _adapted = []

        def _adapter(context, event):
            _adapted.append((context, event))
            return object()
        gsm = getGlobalSiteManager()
        gsm.registerHandler(_adapter, (Interface, IObjectEvent))
        del _adapted[:]  # clear handler reg

        @implementer(IObjectEvent)
        class _ObjectEvent:
            def __init__(self, object):
                self.object = object
        context = object()
        event = _ObjectEvent(context)
        objectEventNotify(event)
        self.assertEqual(_adapted, [(context, event)])



##############################################################################
#
# Copyright (c) 2012 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Tests for z.c.hooks
"""
import unittest

from zope.interface.tests.test_adapter import \
    CustomTypesBaseAdapterRegistryTests


def skipIfNoPersistent(testfunc):
    try:
        import persistent  # noqa: F401 imported but unused
    except ImportError:  # pragma: no cover
        return unittest.skip("persistent not installed")(testfunc)
    return testfunc


@skipIfNoPersistent
class PersistentAdapterRegistryTests(unittest.TestCase):

    def _getTargetClass(self):
        from zope.component.persistentregistry import PersistentAdapterRegistry
        return PersistentAdapterRegistry

    def _makeOne(self, *args, **kw):
        return self._getTargetClass()(*args, **kw)

    def _makeCache(self, jar):
        # Borrowed from persistent.tests.test_pyPersistence.

        class _Cache:
            def __init__(self, jar):
                self._jar = jar
                self._mru = []
                # mru(oid) is only called in pure-Python runs
                self.mru = self._mru.append

            def new_ghost(self, oid, obj):
                obj._p_jar = self._jar
                obj._p_oid = oid

            def update_object_size_estimation(self, oid, size):
                "This is only called in pure-Python runs"

        return _Cache(jar)

    def _makeJar(self):
        # Borrowed from persistent.tests.test_pyPersistence.
        from persistent.interfaces import IPersistentDataManager
        from zope.interface import implementer

        @implementer(IPersistentDataManager)
        class _Jar:
            def __init__(self):
                self._loaded = []
                self._registered = []

            def setstate(self, obj):
                self._loaded.append(obj._p_oid)

            def register(self, obj):
                self._registered.append(obj._p_oid)

        jar = _Jar()
        jar._cache = self._makeCache(jar)
        return jar

    def _makeOneWithJar(self, dirty=False, **kw):
        # Borrowed from persistent.tests.test_pyPersistence.
        OID = _makeOctets('\x01' * 8)
        inst = self._makeOne(**kw)
        jar = self._makeJar()
        jar._cache.new_ghost(OID, inst)  # assigns _p_jar, _p_oid
        return inst, jar, OID

    def test_changed_original_is_not_us(self):
        registry, jar, OID = self._makeOneWithJar()
        self.assertEqual(registry._generation, 1)
        registry.changed(object())
        # 'originally_changed' is not us, but we are still dirty because
        # '_generation' gets bumped.
        self.assertEqual(registry._p_changed, True)
        # base class gets called
        self.assertEqual(registry._generation, 2)

    def test_changed_original_is_us(self):
        registry, jar, OID = self._makeOneWithJar()
        self.assertEqual(registry._generation, 1)
        registry.changed(registry)
        # 'originally_changed' is not us, so not dirty
        self.assertEqual(registry._p_changed, True)
        # base class gets called
        self.assertEqual(registry._generation, 2)

    def test___getstate___simple(self):
        from zope.component import globalSiteManager
        bases = (globalSiteManager.adapters, globalSiteManager.utilities)
        registry, jar, OID = self._makeOneWithJar(bases=bases)
        state = registry.__getstate__()
        self.assertEqual(state.pop('__bases__'), bases)
        self.assertEqual(state.pop('_generation'), 1)
        self.assertEqual(state.pop('_provided'), {})
        self.assertEqual(state.pop('_adapters'), [])
        self.assertEqual(state.pop('_subscribers'), [])
        self.assertNotIn('ro', state)
        self.assertEqual(state, {})

    def test___getstate___skips_delegated_names(self):
        registry, jar, OID = self._makeOneWithJar()
        registry.names = lambda *args: ['a', 'b', 'c']
        self.assertFalse('names' in registry.__getstate__())

    def test___setstate___rebuilds__v_lookup(self):
        registry, jar, OID = self._makeOneWithJar()
        state = registry.__getstate__()
        self.assertTrue('_v_lookup' in registry.__dict__)
        registry._p_changed = None  # clears volatile '_v_lookup'
        self.assertFalse('_v_lookup' in registry.__dict__)
        registry.__setstate__(state)
        self.assertTrue('_v_lookup' in registry.__dict__)

    def test___setstate___rebuilds__ro(self):
        from zope.component import globalSiteManager
        bases = (globalSiteManager.adapters, globalSiteManager.utilities)
        registry, jar, OID = self._makeOneWithJar(bases=bases)
        state = registry.__getstate__()
        registry.__setstate__(state)
        self.assertEqual(registry.__bases__, bases)
        self.assertEqual(registry.ro, [registry] + list(bases))

    def test__addValueToLeaf_existing_is_tuple_converts(self):
        from persistent.list import PersistentList
        registry = self._makeOne()
        # It converts when the tuple is not empty...
        result = registry._addValueToLeaf(('a',), 'b')
        self.assertIsInstance(result, PersistentList)
        self.assertEqual(result, ['a', 'b'])
        # ...and when it is empty...
        result = registry._addValueToLeaf((), 'b')
        self.assertIsInstance(result, PersistentList)
        self.assertEqual(result, ['b'])
        # ...and in fact when it is even missing
        result = registry._addValueToLeaf(None, 'b')
        self.assertIsInstance(result, PersistentList)
        self.assertEqual(result, ['b'])

    def test__removeValueFromLeaf_existing_is_tuple_converts(self):
        from persistent.list import PersistentList
        registry = self._makeOne()
        # It converts when the item is found...
        result = registry._removeValueFromLeaf(('a', 'b'), 'b')
        self.assertIsInstance(result, PersistentList)
        self.assertEqual(result, ['a'])
        # ...and when it is not found
        result = registry._removeValueFromLeaf(('a',), 'b')
        self.assertIsInstance(result, PersistentList)
        self.assertEqual(result, ['a'])

    def test__addValueFromLeaf_preserves_identity(self):
        registry = self._makeOne()
        first = registry._addValueToLeaf(None, 'a')
        second = registry._addValueToLeaf(first, 'b')
        self.assertIs(first, second)
        self.assertEqual(second, ['a', 'b'])

    def test__removeValueFromLeaf_preserves_identity(self):
        registry = self._makeOne()
        first = registry._addValueToLeaf(None, 'a')
        second = registry._addValueToLeaf(first, 'b')
        third = registry._addValueToLeaf(second, 'c')
        fourth = registry._removeValueFromLeaf(third, 'c')
        self.assertIs(first, second)
        self.assertIs(third, fourth)
        self.assertIs(first, fourth)
        self.assertEqual(fourth, ['a', 'b'])


@skipIfNoPersistent
class PersistentComponentsTests(unittest.TestCase):

    def _getTargetClass(self):
        from zope.component.persistentregistry import PersistentComponents
        return PersistentComponents

    def _makeOne(self, *args, **kw):
        return self._getTargetClass()(*args, **kw)

    def test_ctor_initializes_registries_and_registrations(self):
        from persistent.list import PersistentList
        from persistent.mapping import PersistentMapping

        from zope.component.persistentregistry import PersistentAdapterRegistry
        registry = self._makeOne()
        self.assertTrue(isinstance(registry.adapters,
                                   PersistentAdapterRegistry))
        self.assertTrue(isinstance(registry.utilities,
                                   PersistentAdapterRegistry))
        self.assertTrue(isinstance(registry._adapter_registrations,
                                   PersistentMapping))
        self.assertTrue(isinstance(registry._utility_registrations,
                                   PersistentMapping))
        self.assertTrue(isinstance(registry._subscription_registrations,
                                   PersistentList))
        self.assertTrue(isinstance(registry._handler_registrations,
                                   PersistentList))


def _makeOctets(s):
    return bytes(s) if bytes is str else bytes(s, 'ascii')


@skipIfNoPersistent
class PersistentAdapterRegistryCustomTypesTest(
        CustomTypesBaseAdapterRegistryTests):

    def _getMappingType(self):
        from persistent.mapping import PersistentMapping
        return PersistentMapping

    def _getProvidedType(self):
        return self._getMappingType()

    def _getMutableListType(self):
        from persistent.list import PersistentList
        return PersistentList

    def _getLeafSequenceType(self):
        return self._getMutableListType()

    def _getBaseAdapterRegistry(self):
        from zope.component.persistentregistry import PersistentAdapterRegistry
        return PersistentAdapterRegistry



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Zope 3 Component Architecture
"""
import sys

import zope.interface.interface
from zope.hookable import hookable
from zope.interface import Interface
from zope.interface.interfaces import ComponentLookupError
from zope.interface.interfaces import IComponentLookup

from zope.component.interfaces import IFactory
from zope.component.interfaces import inherits_arch_docs as inherits_docs


# getSiteManager() returns a component registry.  Although the term
# "site manager" is deprecated in favor of "component registry",
# the old term is kept around to maintain a stable API.
base = None


@hookable
@inherits_docs
def getSiteManager(context=None):
    """ See IComponentArchitecture.
    """
    global base
    if context is None:
        if base is None:
            from zope.component.globalregistry import base
        return base
    else:
        # Use the global site manager to adapt context to `IComponentLookup`
        # to avoid the recursion implied by using a local `getAdapter()` call.
        try:
            return IComponentLookup(context)
        except TypeError as error:
            raise ComponentLookupError(*error.args)

# Adapter API


@inherits_docs
def getAdapterInContext(object, interface, context):
    adapter = queryAdapterInContext(object, interface, context)
    if adapter is None:
        raise ComponentLookupError(object, interface)
    return adapter


@inherits_docs
def queryAdapterInContext(object, interface, context, default=None):
    conform = getattr(object, '__conform__', None)
    if conform is not None:
        try:
            adapter = conform(interface)
        except TypeError:
            # We got a TypeError. It might be an error raised by
            # the __conform__ implementation, or *we* may have
            # made the TypeError by calling an unbound method
            # (object is a class).  In the later case, we behave
            # as though there is no __conform__ method. We can
            # detect this case by checking whether there is more
            # than one traceback object in the traceback chain:
            if sys.exc_info()[2].tb_next is not None:
                # There is more than one entry in the chain, so
                # reraise the error:
                raise
            # This clever trick is from Phillip Eby
        else:
            if adapter is not None:
                return adapter

    if interface.providedBy(object):
        return object

    return getSiteManager(context).queryAdapter(object, interface, '', default)


@inherits_docs
def getAdapter(object, interface=Interface, name='', context=None):
    adapter = queryAdapter(object, interface, name, None, context)
    if adapter is None:
        raise ComponentLookupError(object, interface, name)
    return adapter


@inherits_docs
def queryAdapter(object, interface=Interface, name='', default=None,
                 context=None):
    if context is None:
        return adapter_hook(interface, object, name, default)
    return getSiteManager(context).queryAdapter(object, interface, name,
                                                default)


@inherits_docs
def getMultiAdapter(objects, interface=Interface, name='', context=None):
    adapter = queryMultiAdapter(objects, interface, name, context=context)
    if adapter is None:
        raise ComponentLookupError(objects, interface, name)
    return adapter


@inherits_docs
def queryMultiAdapter(objects, interface=Interface, name='', default=None,
                      context=None):
    try:
        sitemanager = getSiteManager(context)
    except ComponentLookupError:
        # Oh blast, no site manager. This should *never* happen!
        return default

    return sitemanager.queryMultiAdapter(objects, interface, name, default)


@inherits_docs
def getAdapters(objects, provided, context=None):
    try:
        sitemanager = getSiteManager(context)
    except ComponentLookupError:
        # Oh blast, no site manager. This should *never* happen!
        return []
    return sitemanager.getAdapters(objects, provided)


@inherits_docs
def subscribers(objects, interface, context=None):
    try:
        sitemanager = getSiteManager(context)
    except ComponentLookupError:
        # Oh blast, no site manager. This should *never* happen!
        return []
    return sitemanager.subscribers(objects, interface)


@inherits_docs
def handle(*objects):
    getSiteManager(None).subscribers(objects, None)

#############################################################################
# Register the component architectures adapter hook, with the adapter hook
# registry of the `zope.inteface` package. This way we will be able to call
# interfaces to create adapters for objects. For example, `I1(ob)` is
# equvalent to `getAdapterInContext(I1, ob, '')`.


@hookable
def adapter_hook(interface, object, name='', default=None):
    try:
        sitemanager = getSiteManager()
    except ComponentLookupError:  # pragma: no cover w/o context, cannot test
        # Oh blast, no site manager. This should *never* happen!
        return None
    return sitemanager.queryAdapter(object, interface, name, default)


zope.interface.interface.adapter_hooks.append(adapter_hook)
#############################################################################


# Utility API
@inherits_docs
def getUtility(interface, name='', context=None):
    utility = queryUtility(interface, name, context=context)
    if utility is not None:
        return utility
    raise ComponentLookupError(interface, name)


@inherits_docs
def queryUtility(interface, name='', default=None, context=None):
    return getSiteManager(context).queryUtility(interface, name, default)


@inherits_docs
def getUtilitiesFor(interface, context=None):
    return getSiteManager(context).getUtilitiesFor(interface)


@inherits_docs
def getAllUtilitiesRegisteredFor(interface, context=None):
    return getSiteManager(context).getAllUtilitiesRegisteredFor(interface)


_marker = object()


@inherits_docs
def queryNextUtility(context, interface, name='', default=None):
    """Query for the next available utility.

    Find the next available utility providing `interface` and having the
    specified name. If no utility was found, return the specified `default`
    value.
    """
    try:
        sm = getSiteManager(context)
    except ComponentLookupError:
        return default
    bases = sm.__bases__
    for base in bases:
        util = base.queryUtility(interface, name, _marker)
        if util is not _marker:
            return util
    return default


@inherits_docs
def getNextUtility(context, interface, name=''):
    """Get the next available utility.

    If no utility was found, a `ComponentLookupError` is raised.
    """
    util = queryNextUtility(context, interface, name, _marker)
    if util is _marker:
        raise ComponentLookupError(
            "No more utilities for {}, '{}' have been found.".format(
                interface, name))
    return util


# Factories

@inherits_docs
def createObject(__factory_name, *args, **kwargs):
    """Invoke the named factory and return the result.

    ``__factory_name`` is a positional-only argument.
    """
    context = kwargs.pop('context', None)
    return getUtility(IFactory, __factory_name, context)(*args, **kwargs)


@inherits_docs
def getFactoryInterfaces(name, context=None):
    """Return the interface provided by the named factory's objects

    Result might be a single interface. XXX
    """
    return getUtility(IFactory, name, context).getInterfaces()


@inherits_docs
def getFactoriesFor(interface, context=None):
    """Return info on all factories implementing the given interface.
    """
    utils = getSiteManager(context)
    for (name, factory) in utils.getUtilitiesFor(IFactory):
        interfaces = factory.getInterfaces()
        try:
            if interfaces.isOrExtends(interface):
                yield name, factory
        except AttributeError:
            for iface in interfaces:
                if iface.isOrExtends(interface):
                    yield name, factory
                    break



##############################################################################
#
# Copyright (c) 2012 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Tests for z.c.factory
"""
import unittest

from zope.component.tests import fails_if_called


class FactoryTests(unittest.TestCase):

    def _getTargetClass(self):
        from zope.component.factory import Factory
        return Factory

    def _makeOne(self, callable=None, *args, **kw):
        if callable is None:
            callable = fails_if_called(self)
        return self._getTargetClass()(callable, *args, **kw)

    def test_class_conforms_to_IFactory(self):
        from zope.interface.verify import verifyClass

        from zope.component.interfaces import IFactory
        verifyClass(IFactory, self._getTargetClass())

    def test_instance_conforms_to_IFactory(self):
        from zope.interface.verify import verifyObject

        from zope.component.interfaces import IFactory
        verifyObject(IFactory, self._makeOne())

    def test_ctor_defaults(self):
        func = fails_if_called(self)
        factory = self._makeOne(func)
        self.assertEqual(factory._callable, func)
        self.assertEqual(factory.title, '')
        self.assertEqual(factory.description, '')
        self.assertEqual(factory._interfaces, None)

    def test_ctor_expclit(self):
        factory = self._makeOne(fails_if_called(self), 'TITLE', 'DESCRIPTION')
        self.assertEqual(factory.title, 'TITLE')
        self.assertEqual(factory.description, 'DESCRIPTION')

    def test___call___no_args(self):
        _called = []

        def _callable(*args, **kw):
            _called.append((args, kw))
        factory = self._makeOne(_callable)
        factory()
        self.assertEqual(_called, [((), {})])

    def test___call___positional_args(self):
        _called = []

        def _callable(*args, **kw):
            _called.append((args, kw))
        factory = self._makeOne(_callable)
        factory('one', 'two')
        self.assertEqual(_called, [(('one', 'two'), {})])

    def test___call___keyword_args(self):
        _called = []

        def _callable(*args, **kw):
            _called.append((args, kw))
        factory = self._makeOne(_callable)
        factory(foo='bar')
        self.assertEqual(_called, [((), {'foo': 'bar'})])

    def test_getInterfaces_explicit(self):
        from zope.interface import Interface
        from zope.interface import implementer

        class IFoo(Interface):
            pass

        class IBar(Interface):
            pass

        class IBaz(Interface):
            pass
        _callable = fails_if_called(self)
        _callable.__name__ = '_callable'
        _callable = implementer(IBaz)(_callable)
        factory = self._makeOne(_callable, interfaces=(IFoo, IBar))
        spec = factory.getInterfaces()
        self.assertEqual(spec.__name__, '_callable')
        self.assertEqual(list(spec), [IFoo, IBar])

    def test_getInterfaces_implicit(self):
        from zope.interface import Interface
        from zope.interface import implementer

        class IBaz(Interface):
            pass
        _callable = implementer(IBaz)(fails_if_called(self))
        factory = self._makeOne(_callable)
        spec = factory.getInterfaces()
        self.assertEqual(list(spec), [IBaz])



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Sample adapter class for testing
"""

from zope.interface import Interface
from zope.interface import implementer

from zope.component import adapter
from zope.component.testfiles import components


class I1(Interface):
    pass


class I2(Interface):
    pass


class I3(Interface):
    def f1():
        pass  # pragma: no cover

    def f2():
        pass  # pragma: no cover

    def f3():
        pass  # pragma: no cover


class IS(Interface):
    pass


class Adapter:
    def __init__(self, *args):
        self.context = args


@implementer(I1)
class A1(Adapter):
    pass


@implementer(I2)
class A2(Adapter):
    pass


@adapter(components.IContent, I1, I2)
@implementer(I3)
class A3(Adapter):
    pass


class A4:
    pass


a4 = A4()


@implementer(I1, I2)
class A5:
    pass


a5 = A5()


def Handler(content, *args):
    # uninteresting handler
    content.args = getattr(content, 'args', ()) + (args, )



##############################################################################
#
# Copyright (c) 2012 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Tests for z.c.hooks
"""
import unittest


class Test_read_property(unittest.TestCase):

    def _getTargetClass(self):
        from zope.component.hooks import read_property
        return read_property

    def test_via_instance(self):
        class Foo:
            @self._getTargetClass()
            def bar(self):
                return 'BAR'
        foo = Foo()
        self.assertEqual(foo.bar, 'BAR')
        foo.bar = 'BAZ'
        self.assertEqual(foo.bar, 'BAZ')
        del foo.bar
        self.assertEqual(foo.bar, 'BAR')

    def test_via_class(self):
        class Foo:
            @self._getTargetClass()
            def bar(self):
                return 'BAR'
        bar = Foo.bar
        self.assertTrue(isinstance(bar, self._getTargetClass()))
        self.assertEqual(bar.func(object()), 'BAR')


class SiteInfoTests(unittest.TestCase):

    def _getTargetClass(self):
        from zope.component.hooks import SiteInfo
        return SiteInfo

    def _makeOne(self):
        return self._getTargetClass()()

    def test_initial(self):
        from zope.component.globalregistry import getGlobalSiteManager
        gsm = getGlobalSiteManager()
        si = self._makeOne()
        self.assertEqual(si.site, None)
        self.assertTrue(si.sm is gsm)

    def test_adapter_hook(self):
        _hook = object()

        class _Registry:
            adapter_hook = _hook

        class _SiteManager:
            adapters = _Registry()
        si = self._makeOne()
        si.sm = _SiteManager()
        self.assertFalse('adapter_hook' in si.__dict__)
        self.assertTrue(si.adapter_hook is _hook)
        self.assertTrue('adapter_hook' in si.__dict__)
        del si.adapter_hook
        self.assertFalse('adapter_hook' in si.__dict__)


class Test_setSite(unittest.TestCase):

    def _callFUT(self, site):
        from zope.component.hooks import setSite
        return setSite(site)

    def test_w_None(self):
        from zope.component import hooks
        from zope.component.globalregistry import getGlobalSiteManager
        gsm = getGlobalSiteManager()
        _SM2 = object()
        _SITE = object()
        _HOOK = object()
        siteinfo = _DummySiteInfo()
        siteinfo.sm = _SM2
        siteinfo.site = _SITE
        siteinfo.adapterhook = _HOOK
        with _Monkey(hooks, siteinfo=siteinfo):
            self._callFUT(None)
        self.assertTrue(siteinfo.sm is gsm)
        self.assertTrue(siteinfo.site is None)
        self.assertFalse('adapter_hook' in siteinfo.__dict__)

    def test_w_site(self):
        from zope.component import hooks
        _SM2 = object()

        class _Site:
            def getSiteManager(self):
                return _SM2
        siteinfo = _DummySiteInfo()
        _site = _Site()
        with _Monkey(hooks, siteinfo=siteinfo):
            self._callFUT(_site)
        self.assertTrue(siteinfo.sm is _SM2)
        self.assertTrue(siteinfo.site is _site)
        self.assertFalse('adapter_hook' in siteinfo.__dict__)


class Test_getSite(unittest.TestCase):

    def _callFUT(self):
        from zope.component.hooks import getSite
        return getSite()

    def test_w_None(self):
        from zope.component import hooks
        siteinfo = _DummySiteInfo()
        with _Monkey(hooks, siteinfo=siteinfo):
            self.assertTrue(self._callFUT() is None)

    def test_w_site(self):
        from zope.component import hooks
        _SM2 = object()
        _SITE = object()
        siteinfo = _DummySiteInfo()
        siteinfo.sm = _SM2
        siteinfo.site = _SITE
        with _Monkey(hooks, siteinfo=siteinfo):
            self.assertTrue(self._callFUT() is _SITE)


class Test_site(unittest.TestCase):

    def _callFUT(self, new_site):
        from zope.component.hooks import site
        return site(new_site)

    def test_it(self):
        from zope.component import hooks
        from zope.component.globalregistry import getGlobalSiteManager
        gsm = getGlobalSiteManager()
        _SM2 = object()

        class _Site:
            def getSiteManager(self):
                return _SM2
        _site = _Site()
        siteinfo = _DummySiteInfo()
        self.assertTrue(siteinfo.site is None)
        self.assertTrue(siteinfo.sm is _SM)
        with _Monkey(hooks, siteinfo=siteinfo):
            with self._callFUT(_site):
                self.assertTrue(siteinfo.site is _site)
                self.assertTrue(siteinfo.sm is _SM2)
            self.assertTrue(siteinfo.site is None)
            self.assertTrue(siteinfo.sm is gsm)


class Test_getSiteManager(unittest.TestCase):

    def _callFUT(self, context=None):
        from zope.component.hooks import getSiteManager
        return getSiteManager(context)

    def test_default(self):
        from zope.component import hooks
        _SM2 = object()
        siteinfo = _DummySiteInfo()
        siteinfo.sm = _SM2
        with _Monkey(hooks, siteinfo=siteinfo):
            self.assertTrue(self._callFUT() is _SM2)

    def test_w_explicit_context_no_IComponentLookup(self):
        from zope.component import hooks
        from zope.component.globalregistry import getGlobalSiteManager
        gsm = getGlobalSiteManager()
        _SM2 = object()
        siteinfo = _DummySiteInfo()
        siteinfo.sm = _SM2
        with _Monkey(hooks, siteinfo=siteinfo):
            self.assertTrue(self._callFUT(object()) is gsm)

    def test_w_explicit_context_w_IComponentLookup(self):
        from zope.interface import Interface
        from zope.interface.interfaces import IComponentLookup

        from zope.component import hooks
        from zope.component.globalregistry import getGlobalSiteManager

        class _Lookup:
            def __init__(self, context):
                self.context = context
        gsm = getGlobalSiteManager()
        gsm.registerAdapter(_Lookup, (Interface,), IComponentLookup, '')
        _SM2 = object()
        siteinfo = _DummySiteInfo()
        siteinfo.sm = _SM2
        context = object()
        with _Monkey(hooks, siteinfo=siteinfo):
            sm = self._callFUT(context)
        self.assertTrue(isinstance(sm, _Lookup))
        self.assertTrue(sm.context is context)


class Test_adapter_hook(unittest.TestCase):

    def _callFUT(self, interface, object, name='', default=None):
        from zope.component.hooks import adapter_hook
        return adapter_hook(interface, object, name, default)

    def test_success(self):
        from zope.interface import Interface

        from zope.component import hooks

        class IFoo(Interface):
            pass
        _ADAPTER = object()
        _DEFAULT = object()
        _CONTEXT = object()
        _called = []

        def _adapter_hook(interface, object, name, default):
            _called.append((interface, object, name, default))
            return _ADAPTER
        siteinfo = _DummySiteInfo()
        siteinfo.adapter_hook = _adapter_hook
        with _Monkey(hooks, siteinfo=siteinfo):
            adapter = self._callFUT(IFoo, _CONTEXT, 'bar', _DEFAULT)
        self.assertTrue(adapter is _ADAPTER)
        self.assertEqual(_called, [(IFoo, _CONTEXT, 'bar', _DEFAULT)])

    def test_hook_raises(self):
        from zope.interface import Interface
        from zope.interface.interfaces import ComponentLookupError

        from zope.component import hooks

        class IFoo(Interface):
            pass
        _DEFAULT = object()
        _CONTEXT = object()
        _called = []

        def _adapter_hook(interface, object, name, default):
            _called.append((interface, object, name, default))
            raise ComponentLookupError('testing')
        siteinfo = _DummySiteInfo()
        siteinfo.adapter_hook = _adapter_hook
        with _Monkey(hooks, siteinfo=siteinfo):
            adapter = self._callFUT(IFoo, _CONTEXT, 'bar', _DEFAULT)
        self.assertTrue(adapter is _DEFAULT)
        self.assertEqual(_called, [(IFoo, _CONTEXT, 'bar', _DEFAULT)])


class Test_setHooks(unittest.TestCase):

    def _callFUT(self):
        from zope.component.hooks import setHooks
        return setHooks()

    def test_it(self):
        import zope.component._api
        from zope.component import hooks

        class _Hook:
            def __init__(self):
                self._hooked = None

            def sethook(self, value):
                self._hooked = value
        adapter_hook = _Hook()
        getSiteManager = _Hook()
        with _Monkey(zope.component._api,
                     adapter_hook=adapter_hook,
                     getSiteManager=getSiteManager):
            self._callFUT()
        self.assertEqual(adapter_hook._hooked, hooks.adapter_hook)
        self.assertEqual(getSiteManager._hooked, hooks.getSiteManager)


class Test_resetHooks(unittest.TestCase):

    def _callFUT(self):
        from zope.component.hooks import resetHooks
        return resetHooks()

    def test_it(self):
        import zope.component._api
        from zope.component import hooks

        class _Hook:
            def __init__(self):
                self._reset = False

            def reset(self):
                self._reset = True
        adapter_hook = _Hook()
        getSiteManager = _Hook()
        with _Monkey(zope.component._api,
                     adapter_hook=adapter_hook,
                     getSiteManager=getSiteManager):
            # Access the adapter_hook of the site info to be
            # sure it caches
            getattr(hooks.siteinfo, 'adapter_hook')
            self.assertTrue('adapter_hook' in hooks.siteinfo.__dict__)

            self._callFUT()

        self.assertTrue(adapter_hook._reset)
        self.assertTrue(getSiteManager._reset)
        # adapter_hook cache also reset
        self.assertFalse('adapter_hook' in hooks.siteinfo.__dict__)


_SM = object()


class _DummySiteInfo:
    sm = _SM
    site = None


class _Monkey:
    # context-manager for replacing module names in the scope of a test.
    def __init__(self, module, **kw):
        self.module = module
        self.to_restore = {key: getattr(module, key) for key in kw}
        for key, value in kw.items():
            setattr(module, key, value)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        for key, value in self.to_restore.items():
            setattr(self.module, key, value)



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Interface utility functions
"""
from zope.interface import alsoProvides
from zope.interface.interfaces import ComponentLookupError
from zope.interface.interfaces import IInterface

from zope.component._api import getSiteManager
from zope.component._api import queryUtility


def provideInterface(id, interface, iface_type=None, info=''):
    """
    Mark *interface* as a named utility providing *iface_type*'.

    .. versionchanged:: 5.0.0
       The named utility is registered in the current site manager.
       Previously it was always registered in the global site manager.
    """
    if not id:
        id = "{}.{}".format(interface.__module__, interface.__name__)

    if not IInterface.providedBy(interface):
        if not isinstance(interface, type):
            raise TypeError(id, "is not an interface or class")
        return

    if iface_type is not None:
        if not iface_type.extends(IInterface):
            raise TypeError(iface_type, "is not an interface type")
        alsoProvides(interface, iface_type)
    else:
        iface_type = IInterface

    site_man = getSiteManager()
    site_man.registerUtility(interface, iface_type, id, info)


def getInterface(context, id):
    """Return interface or raise ComponentLookupError
    """
    iface = queryInterface(id, None)
    if iface is None:
        raise ComponentLookupError(id)
    return iface


def queryInterface(id, default=None):
    """Return an interface or ``None``
    """
    return queryUtility(IInterface, id, default)


def searchInterface(context, search_string=None, base=None):
    """Interfaces search
    """
    return [iface_util[1] for iface_util in
            searchInterfaceUtilities(context, search_string, base)]


def searchInterfaceIds(context, search_string=None, base=None):
    """Interfaces search
    """
    return [iface_util[0] for iface_util in
            searchInterfaceUtilities(context, search_string, base)]


def searchInterfaceUtilities(context, search_string=None, base=None):
    site_man = getSiteManager()
    iface_utilities = site_man.getUtilitiesFor(IInterface)

    if search_string:
        search_string = search_string.lower()
        iface_utilities = [iface_util for iface_util in iface_utilities
                           if (getInterfaceAllDocs(iface_util[1]).
                               find(search_string) >= 0)]
    if base:
        res = [iface_util for iface_util in iface_utilities
               if iface_util[1].isOrExtends(base)]
    else:
        res = list(iface_utilities)
    return res


def getInterfaceAllDocs(interface):
    iface_id = '{}.{}'.format(interface.__module__, interface.__name__)
    docs = [str(iface_id).lower(),
            str(interface.__doc__).lower()]

    if IInterface.providedBy(interface):
        for name in sorted(interface):
            docs.append(
                str(interface.getDescriptionFor(name).__doc__).lower())

    return '\n'.join(docs)


def nameToInterface(context, id):
    if id == 'None':
        return None
    iface = getInterface(context, id)
    return iface


def interfaceToName(context, interface):
    if interface is None:
        return 'None'
    return '{}.{}'.format(interface.__module__, interface.__name__)



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Hooks for getting and setting a site in the thread global namespace.
"""
__docformat__ = 'restructuredtext'

import contextlib
import threading

from zope.component._compat import ZOPE_SECURITY_NOT_AVAILABLE_EX


try:
    from zope.security.proxy import removeSecurityProxy
except ZOPE_SECURITY_NOT_AVAILABLE_EX:  # pragma: no cover
    def removeSecurityProxy(x):
        return x

from zope.interface.interfaces import ComponentLookupError
from zope.interface.interfaces import IComponentLookup

from zope.component.globalregistry import getGlobalSiteManager


__all__ = [
    'setSite',
    'getSite',
    'site',
    'getSiteManager',
    'setHooks',
    'resetHooks',
]


class read_property:
    """Descriptor for property-like computed attributes.

    Unlike the standard 'property', this descriptor allows assigning a
    value to the instance, shadowing the property getter function.
    """

    def __init__(self, func):
        self.func = func

    def __get__(self, inst, cls):
        if inst is None:
            return self

        return self.func(inst)


class SiteInfo(threading.local):
    site = None
    sm = getGlobalSiteManager()

    @read_property
    def adapter_hook(self):
        adapter_hook = self.sm.adapters.adapter_hook
        self.adapter_hook = adapter_hook
        return adapter_hook


siteinfo = SiteInfo()


def setSite(site=None):
    if site is None:
        sm = getGlobalSiteManager()
    else:

        # We remove the security proxy because there's no way for
        # untrusted code to get at it without it being proxied again.

        # We should really look look at this again though, especially
        # once site managers do less.  There's probably no good reason why
        # they can't be proxied.  Well, except maybe for performance.

        site = removeSecurityProxy(site)
        # The getSiteManager method is defined by IPossibleSite.
        sm = site.getSiteManager()

    siteinfo.site = site
    siteinfo.sm = sm
    try:
        del siteinfo.adapter_hook
    except AttributeError:
        pass


def getSite():
    return siteinfo.site


@contextlib.contextmanager
def site(site):
    """
    site(site) -> None

    Context manager that sets *site* as the current site for the
    duration of the ``with`` body.
    """
    old_site = getSite()
    setSite(site)
    try:
        yield
    finally:
        setSite(old_site)


def getSiteManager(context=None):
    """A special hook for getting the site manager.

    Here we take the currently set site into account to find the appropriate
    site manager.
    """
    if context is None:
        return siteinfo.sm

    # We remove the security proxy because there's no way for
    # untrusted code to get at it without it being proxied again.

    # We should really look look at this again though, especially
    # once site managers do less.  There's probably no good reason why
    # they can't be proxied.  Well, except maybe for performance.
    sm = IComponentLookup(
        context, getGlobalSiteManager())
    sm = removeSecurityProxy(sm)
    return sm


def adapter_hook(interface, object, name='', default=None):
    try:
        return siteinfo.adapter_hook(interface, object, name, default)
    except ComponentLookupError:
        return default


def setHooks():
    """
    Make `zope.component.getSiteManager` and interface adaptation
    respect the current site.

    Most applications will want to be sure te call this early in their
    startup sequence. Test code that uses these APIs should also arrange to
    call this.

    .. seealso:: :mod:`zope.component.testlayer`
    """
    from zope.component import _api
    _api.adapter_hook.sethook(adapter_hook)
    _api.getSiteManager.sethook(getSiteManager)


def resetHooks():
    """
    Reset `zope.component.getSiteManager` and interface adaptation to
    their original implementations that are unaware of the current
    site.

    Use caution when calling this; most code will not need to call
    this. If code using the global API executes following this, it
    will most likely use the base global component registry instead of
    a site-specific registry it was expected. This can lead to
    failures in adaptation and utility lookup.
    """
    # Reset hookable functions to original implementation.
    from zope.component import _api
    _api.adapter_hook.reset()
    _api.getSiteManager.reset()
    # be sure the old adapter hook isn't cached, since
    # it is derived from the SiteManager
    try:
        del siteinfo.adapter_hook
    except AttributeError:
        pass


# Clear the site thread global
clearSite = setSite
try:
    from zope.testing.cleanup import addCleanUp
except ImportError:  # pragma: no cover
    pass
else:
    addCleanUp(resetHooks)



##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""zope.security support for the configuration handlers
"""
from zope.interface import providedBy
from zope.proxy import ProxyBase
from zope.proxy import getProxiedObject

from zope.component._compat import ZOPE_SECURITY_NOT_AVAILABLE_EX


try:
    from zope.security.adapter import LocatingTrustedAdapterFactory
    from zope.security.adapter import LocatingUntrustedAdapterFactory
    from zope.security.adapter import TrustedAdapterFactory
    from zope.security.checker import Checker
    from zope.security.checker import CheckerPublic
    from zope.security.checker import InterfaceChecker
    from zope.security.proxy import Proxy
except ZOPE_SECURITY_NOT_AVAILABLE_EX:  # pragma: no cover
    def _no_security(*args, **kw):
        raise TypeError(
            "security proxied components are not "
            "supported because zope.security is not available")

    LocatingTrustedAdapterFactory = _no_security
    LocatingUntrustedAdapterFactory = _no_security
    TrustedAdapterFactory = _no_security
    Checker = _no_security
    CheckerPublic = _no_security
    InterfaceChecker = _no_security


PublicPermission = 'zope.Public'


class PermissionProxy(ProxyBase):

    __slots__ = ('__Security_checker__', )

    def __providedBy__(self):
        return providedBy(getProxiedObject(self))
    __providedBy__ = property(__providedBy__)


def _checker(_context, permission, allowed_interface, allowed_attributes):
    if (not allowed_attributes) and (not allowed_interface):
        allowed_attributes = ["__call__"]

    if permission == PublicPermission:
        permission = CheckerPublic

    require = {}
    if allowed_attributes:
        for name in allowed_attributes:
            require[name] = permission
    if allowed_interface:
        for i in allowed_interface:
            for name in i.names(all=True):
                require[name] = permission

    checker = Checker(require)
    return checker


def proxify(ob, checker=None, provides=None, permission=None):
    """Try to get the object proxied with the `checker`, but not too soon

    We really don't want to proxy the object unless we need to.
    """

    if checker is None:
        if provides is None or permission is None:
            raise ValueError('Required arguments: '
                             'checker or both provides and permissions')
        if permission == PublicPermission:
            permission = CheckerPublic
        checker = InterfaceChecker(provides, permission)
    ob = PermissionProxy(ob)
    ob.__Security_checker__ = checker
    return ob


def protectedFactory(original_factory, provides, permission):
    if permission == PublicPermission:
        permission = CheckerPublic
    checker = InterfaceChecker(provides, permission)
    # This has to be named 'factory', aparently, so as not to confuse apidoc :(

    def factory(*args):
        ob = original_factory(*args)
        try:
            ob.__Security_checker__ = checker
        except AttributeError:
            ob = Proxy(ob, checker)
        return ob
    factory.factory = original_factory
    return factory


def securityAdapterFactory(factory, permission, locate, trusted):
    if permission == PublicPermission:
        permission = CheckerPublic
    if locate or (permission is not None and permission is not CheckerPublic):
        if trusted:
            return LocatingTrustedAdapterFactory(factory)
        else:
            return LocatingUntrustedAdapterFactory(factory)
    elif trusted:
        return TrustedAdapterFactory(factory)
    else:
        return factory



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Persistent component managers.
"""
from persistent import Persistent
from persistent.list import PersistentList
from persistent.mapping import PersistentMapping
from zope.interface.adapter import VerifyingAdapterRegistry
from zope.interface.registry import Components


class PersistentAdapterRegistry(VerifyingAdapterRegistry, Persistent):
    """
    An adapter registry that is also a persistent object.

    .. versionchanged:: 5.0.0
        Internal data structures are now composed of
        :class:`persistent.mapping.PersistentMapping` and
        :class:`persistent.list.PersistentList`. This helps scale to
        larger registries.

        Previously they were :class:`dict`, :class:`list` and
        :class:`tuple`, meaning that as soon as this object was
        unpickled, the entire registry tree had to be unpickled, and
        when a change was made (an object registered or unregisterd),
        the entire registry had to be pickled. Now, only the parts
        that are in use are loaded, and only the parts that are
        modified are stored.

        The above applies without reservation to newly created
        instances. For existing persistent instances, however, the
        data will continue to be in dicts and tuples, with some few
        exceptions for newly added or changed data.

        To fix this, call :meth:`rebuild` and commit the transaction.
        This will rewrite the internal data structures to use the new
        types.
    """

    # The persistent types we use, replacing the basic types inherited
    # from ``BaseAdapterRegistry``.
    _sequenceType = PersistentList
    _leafSequenceType = PersistentList
    _mappingType = PersistentMapping
    _providedType = PersistentMapping

    # The methods needed to manipulate the leaves of the subscriber
    # tree. When we're manipulating unmigrated data, it's safe to
    # migrate it, but not otherwise (we don't want to write in an
    # otherwise read-only transaction).
    def _addValueToLeaf(self, existing_leaf_sequence, new_item):
        if not existing_leaf_sequence:
            existing_leaf_sequence = self._leafSequenceType()
        elif isinstance(existing_leaf_sequence, tuple):
            # Converting from old state.
            existing_leaf_sequence = self._leafSequenceType(
                existing_leaf_sequence)
        existing_leaf_sequence.append(new_item)
        return existing_leaf_sequence

    def _removeValueFromLeaf(self, existing_leaf_sequence, to_remove):
        if isinstance(existing_leaf_sequence, tuple):
            # Converting from old state
            existing_leaf_sequence = self._leafSequenceType(
                existing_leaf_sequence)

        without_removed = VerifyingAdapterRegistry._removeValueFromLeaf(
            self,
            existing_leaf_sequence,
            to_remove)
        existing_leaf_sequence[:] = without_removed
        return existing_leaf_sequence

    def changed(self, originally_changed):
        if originally_changed is self:
            # XXX: This is almost certainly redundant, even if we
            # have old data consisting of plain dict/list/tuple. That's
            # because the super class will now increment the ``_generation``
            # attribute to keep caches in sync. For this same reason,
            # it's not worth trying to "optimize" for the case that we're a
            # new or rebuilt object containing only Persistent sub-objects:
            # the changed() mechanism will still result in mutating this
            # object via ``_generation``.
            self._p_changed = True
        super().changed(originally_changed)

    def __getstate__(self):
        state = super().__getstate__().copy()
        for name in self._delegated:
            state.pop(name, 0)
        state.pop('ro', None)
        return state

    def __setstate__(self, state):
        bases = state.pop('__bases__', ())
        super().__setstate__(state)
        self._createLookup()
        self.__bases__ = bases
        self._v_lookup.changed(self)


class PersistentComponents(Components):
    """
    A component implementation that uses `PersistentAdapterRegistry`.

    Note that this object itself is *not* `Persistent`.
    """

    def _init_registries(self):
        self.adapters = PersistentAdapterRegistry()
        self.utilities = PersistentAdapterRegistry()

    def _init_registrations(self):
        self._utility_registrations = PersistentMapping()
        self._adapter_registrations = PersistentMapping()
        self._subscription_registrations = PersistentList()
        self._handler_registrations = PersistentList()



"""
See: https://bugs.launchpad.net/zope3/+bug/98401
"""

import pickle
import sys


def write(x):
    sys.stdout.write('%s\n' % x)


if __name__ == "__main__":
    # TextIO? Are you kidding me?
    data = sys.stdin.buffer.read()
    sys.path = pickle.loads(data)
    write('XXXXXXXXXX')
    for p in sys.path:
        write('- %s' % p)
    write('XXXXXXXXXX')

    from zope.interface import Interface
    from zope.interface import implementer

    import zope

    class I1(Interface):
        pass

    class I2(Interface):
        pass

    @implementer(I1)
    class Ob:
        def __repr__(self):
            return '<instance Ob>'

    ob = Ob()

    @implementer(I2)
    class Comp:
        def __init__(self, context):
            self.context = context

    write('YYYYYYYYY')
    for p in zope.__path__:
        write('- %s' % p)
    write('YYYYYYYYY')
    import zope.component

    zope.component.provideAdapter(Comp, (I1,), I2)
    adapter = I2(ob)
    write('ZZZZZZZZ')
    assert adapter.__class__ is Comp
    assert adapter.context is ob



##############################################################################
#
# Copyright (c) 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Components for testing
"""
from zope.interface import Attribute
from zope.interface import Interface
from zope.interface import implementer

from zope.component import adapter
from zope.component import named


class IAppb(Interface):
    a = Attribute('test attribute')

    def f():
        "test func"


class IApp(IAppb):
    pass


class IApp2(IAppb):
    pass


class IApp3(IAppb):
    pass


class IContent(Interface):
    pass


@implementer(IContent)
class Content:
    pass


@adapter(IContent)
@implementer(IApp)
class Comp:

    def __init__(self, *args):
        # Ignore arguments passed to constructor
        pass

    a = 1

    def f():
        pass  # pragma: no cover


class Comp2:
    def __init__(self, context):
        self.context = context


@adapter(IContent)
@implementer(IApp)
@named('app')
class Comp4:
    def __init__(self, context=None):
        self.context = context


comp = Comp()
comp4 = Comp4()

content = Content()



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Component Architecture-specific event dispatching

Based on subscription adapters / handlers.
"""

from zope.event import subscribers as event_subscribers
from zope.interface.interfaces import IObjectEvent

from zope.component._api import subscribers as component_subscribers
from zope.component._declaration import adapter


def dispatch(*event):
    component_subscribers(event, None)


event_subscribers.append(dispatch)


@adapter(IObjectEvent)
def objectEventNotify(event):
    """Dispatch ObjectEvents to interested adapters.
    """
    component_subscribers((event.object, event), None)



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
Zope 3 Component Architecture

This module provides an implementation of
:class:`~zope.component.interfaces.IComponentArchitecture`, using the
current :mod:`site <zope.component.hooks>`.

This module also provides an implementation of
:class:`~zope.component.interfaces.IComponentRegistrationConvenience`
using the `global site manager <getGlobalSiteManager>`_.
"""
from zope.interface import Interface
from zope.interface import implementedBy
from zope.interface import moduleProvides
from zope.interface import named
from zope.interface import providedBy
from zope.interface.interfaces import ComponentLookupError
from zope.interface.interfaces import IComponentLookup

from zope.component._api import adapter_hook
from zope.component._api import createObject
from zope.component._api import getAdapter
from zope.component._api import getAdapterInContext
from zope.component._api import getAdapters
from zope.component._api import getAllUtilitiesRegisteredFor
from zope.component._api import getFactoriesFor
from zope.component._api import getFactoryInterfaces
from zope.component._api import getMultiAdapter
from zope.component._api import getNextUtility
from zope.component._api import getSiteManager
from zope.component._api import getUtilitiesFor
from zope.component._api import getUtility
from zope.component._api import handle
from zope.component._api import queryAdapter
from zope.component._api import queryAdapterInContext
from zope.component._api import queryMultiAdapter
from zope.component._api import queryNextUtility
from zope.component._api import queryUtility
from zope.component._api import subscribers
from zope.component._declaration import adaptedBy
from zope.component._declaration import adapter
from zope.component._declaration import adapts
from zope.component.globalregistry import getGlobalSiteManager
from zope.component.globalregistry import globalSiteManager
from zope.component.globalregistry import provideAdapter
from zope.component.globalregistry import provideHandler
from zope.component.globalregistry import provideSubscriptionAdapter
from zope.component.globalregistry import provideUtility
from zope.component.interfaces import IComponentArchitecture
from zope.component.interfaces import IComponentRegistrationConvenience
from zope.component.interfaces import IFactory


moduleProvides(IComponentArchitecture, IComponentRegistrationConvenience)
__all__ = tuple(IComponentArchitecture) \
    + tuple(IComponentRegistrationConvenience) \
    + ('named', 'adapts', 'adapter', 'adaptedBy', )



##############################################################################
#
# Copyright (c) 2012 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Tests for z.c._api
"""
import unittest

from zope.component.tests import fails_if_called


class Test_getGlobalSiteManager(unittest.TestCase):

    def _callFUT(self):
        from zope.component.globalregistry import getGlobalSiteManager
        return getGlobalSiteManager()

    def test_gsm_is_IComponentLookup(self):
        from zope.interface.interfaces import IComponentLookup

        from zope.component.globalregistry import base
        gsm = self._callFUT()
        self.assertTrue(gsm is base)
        self.assertTrue(IComponentLookup.providedBy(gsm))

    def test_gsm_is_singleton(self):
        gsm = self._callFUT()
        self.assertTrue(self._callFUT() is gsm)

    def test_gsm_pickling(self):
        import pickle
        gsm = self._callFUT()
        dumped = pickle.dumps(gsm)
        loaded = pickle.loads(dumped)
        self.assertTrue(loaded is gsm)

        dumped_utilities = pickle.dumps(gsm.utilities)
        loaded_utilities = pickle.loads(dumped_utilities)
        self.assertTrue(loaded_utilities is gsm.utilities)

        dumped_adapters = pickle.dumps(gsm.adapters)
        loaded_adapters = pickle.loads(dumped_adapters)
        self.assertTrue(loaded_adapters is gsm.adapters)


class Test_provideUtility(unittest.TestCase):

    from zope.component.testing import setUp
    from zope.component.testing import tearDown

    def _callFUT(self, *args, **kw):
        from zope.component.globalregistry import provideUtility
        return provideUtility(*args, **kw)

    def test_anonymous_no_provides(self):
        from zope.interface import Interface
        from zope.interface import implementer

        from zope.component.globalregistry import getGlobalSiteManager

        class IFoo(Interface):
            pass

        @implementer(IFoo)
        class Foo:
            pass
        foo = Foo()
        self._callFUT(foo)
        gsm = getGlobalSiteManager()
        self.assertIs(gsm.getUtility(IFoo, ''), foo)

        # We can clean it up using the fallback and it will be gone
        from zope.component.testing import _PlacelessSetupFallback
        _PlacelessSetupFallback().cleanUp()
        self.assertIsNone(gsm.queryUtility(IFoo, ''))

    def test_named_w_provides(self):
        from zope.interface import Interface

        from zope.component.globalregistry import getGlobalSiteManager

        class IFoo(Interface):
            pass

        class Foo:
            pass
        foo = Foo()
        self._callFUT(foo, IFoo, 'named')
        gsm = getGlobalSiteManager()
        self.assertTrue(gsm.getUtility(IFoo, 'named') is foo)


class Test_provideAdapter(unittest.TestCase):

    from zope.component.testing import setUp
    from zope.component.testing import tearDown

    def _callFUT(self, *args, **kw):
        from zope.component.globalregistry import provideAdapter
        return provideAdapter(*args, **kw)

    def test_anonymous_no_provides_no_adapts(self):
        from zope.interface import Interface
        from zope.interface import implementer

        from zope.component._declaration import adapter
        from zope.component.globalregistry import getGlobalSiteManager

        class IFoo(Interface):
            pass

        class IBar(Interface):
            pass

        @implementer(IFoo)
        class Foo:
            pass

        @adapter(IFoo)
        @implementer(IBar)
        class Bar:
            def __init__(self, context):
                self.context = context
        self._callFUT(Bar)
        gsm = getGlobalSiteManager()
        foo = Foo()
        adapted = gsm.getAdapter(foo, IBar)
        self.assertTrue(isinstance(adapted, Bar))
        self.assertTrue(adapted.context is foo)

    def test_named_w_provides_w_adapts(self):
        from zope.interface import Interface
        from zope.interface import implementer

        from zope.component.globalregistry import getGlobalSiteManager

        class IFoo(Interface):
            pass

        class IBar(Interface):
            pass

        @implementer(IFoo)
        class Foo:
            pass

        class Bar:
            def __init__(self, context):
                self.context = context
        self._callFUT(Bar, (IFoo,), IBar, 'test')
        gsm = getGlobalSiteManager()
        foo = Foo()
        adapted = gsm.getAdapter(foo, IBar, name='test')
        self.assertTrue(isinstance(adapted, Bar))
        self.assertTrue(adapted.context is foo)


class Test_provideSubscriptionAdapter(unittest.TestCase):

    from zope.component.testing import setUp
    from zope.component.testing import tearDown

    def _callFUT(self, *args, **kw):
        from zope.component.globalregistry import provideSubscriptionAdapter
        return provideSubscriptionAdapter(*args, **kw)

    def test_no_provides_no_adapts(self):
        from zope.interface import Interface
        from zope.interface import implementer

        from zope.component._declaration import adapter
        from zope.component.globalregistry import getGlobalSiteManager

        class IFoo(Interface):
            pass

        class IBar(Interface):
            pass

        @implementer(IFoo)
        class Foo:
            pass

        @adapter(IFoo)
        @implementer(IBar)
        class Bar:
            def __init__(self, context):
                self.context = context
        self._callFUT(Bar)
        gsm = getGlobalSiteManager()
        foo = Foo()
        adapted = gsm.subscribers((foo,), IBar)
        self.assertEqual(len(adapted), 1)
        self.assertTrue(isinstance(adapted[0], Bar))
        self.assertTrue(adapted[0].context is foo)

    def test_w_provides_w_adapts(self):
        from zope.interface import Interface
        from zope.interface import implementer

        from zope.component.globalregistry import getGlobalSiteManager

        class IFoo(Interface):
            pass

        class IBar(Interface):
            pass

        @implementer(IFoo)
        class Foo:
            pass

        class Bar:
            def __init__(self, context):
                self.context = context
        self._callFUT(Bar, (IFoo,), IBar)
        gsm = getGlobalSiteManager()
        foo = Foo()
        adapted = gsm.subscribers((foo,), IBar)
        self.assertEqual(len(adapted), 1)
        self.assertTrue(isinstance(adapted[0], Bar))
        self.assertTrue(adapted[0].context is foo)


class Test_provideHandler(unittest.TestCase):

    from zope.component.testing import setUp
    from zope.component.testing import tearDown

    def _callFUT(self, *args, **kw):
        from zope.component.globalregistry import provideHandler
        return provideHandler(*args, **kw)

    def test_no_adapts(self):
        from zope.interface import Interface
        from zope.interface import implementer
        from zope.interface import providedBy

        from zope.component._declaration import adapter
        from zope.component.globalregistry import getGlobalSiteManager

        class IFoo(Interface):
            pass

        @implementer(IFoo)
        class Foo:
            pass
        _handler = adapter(IFoo)(fails_if_called(self))

        self._callFUT(_handler)
        gsm = getGlobalSiteManager()
        regs = list(gsm.registeredHandlers())
        self.assertEqual(len(regs), 1)
        hr = regs[0]
        self.assertEqual(list(hr.required), list(providedBy(Foo())))
        self.assertEqual(hr.name, '')
        self.assertTrue(hr.factory is _handler)

    def test_w_adapts(self):
        from zope.interface import Interface

        from zope.component.globalregistry import getGlobalSiteManager

        class IFoo(Interface):
            pass
        _handler = fails_if_called(self)
        self._callFUT(_handler, (IFoo,))
        gsm = getGlobalSiteManager()
        regs = list(gsm.registeredHandlers())
        self.assertEqual(len(regs), 1)
        hr = regs[0]
        self.assertEqual(list(hr.required), [IFoo])
        self.assertEqual(hr.name, '')
        self.assertTrue(hr.factory is _handler)


class TestBaseGlobalComponents(unittest.TestCase):

    from zope.component.testing import setUp
    from zope.component.testing import tearDown

    def _getTargetClass(self):
        from zope.component.globalregistry import BaseGlobalComponents
        return BaseGlobalComponents

    def _getTargetInterfaces(self):
        from zope.interface.interfaces import IComponentLookup
        from zope.interface.interfaces import IComponentRegistry
        from zope.interface.interfaces import IComponents
        return [IComponents, IComponentLookup, IComponentRegistry]

    def _makeOne(self):
        return self._getTargetClass()()

    def test_verifyInstance(self):
        from zope.interface.verify import verifyObject
        for iface in self._getTargetInterfaces():
            verifyObject(iface, self._makeOne())

    def test_verifyClass(self):
        from zope.interface.verify import verifyClass
        for iface in self._getTargetInterfaces():
            verifyClass(iface, self._getTargetClass())



##############################################################################
#
# Copyright (c) 2012 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

import unittest


class Test_package(unittest.TestCase):

    def test_module_conforms_to_IComponentArchitecture(self):
        from zope.interface.verify import verifyObject

        import zope.component as zc
        from zope.component.interfaces import IComponentArchitecture
        verifyObject(IComponentArchitecture, zc)

    def test_module_conforms_to_IComponentRegistrationConvenience(self):
        from zope.interface.verify import verifyObject

        import zope.component as zc
        from zope.component.interfaces import IComponentRegistrationConvenience
        verifyObject(IComponentRegistrationConvenience, zc)


class Test_Interface_call(unittest.TestCase):

    from zope.component.testing import setUp
    from zope.component.testing import tearDown

    def test_miss(self):
        from zope.interface import Interface

        class IFoo(Interface):
            pass
        self.assertRaises(TypeError, IFoo, object())

    def test_miss_w_default(self):
        from zope.interface import Interface

        class IFoo(Interface):
            pass
        marker = object()
        self.assertTrue(IFoo(object(), marker) is marker)

    def test_hit(self):
        from zope.interface import Interface
        from zope.interface import implementer

        from zope.component import getGlobalSiteManager

        class IFoo(Interface):
            pass

        class IBar(Interface):
            pass

        @implementer(IBar)
        class Bar:
            pass

        @implementer(IFoo)
        class Baz:
            def __init__(self, context):
                self.context = context
        getGlobalSiteManager().registerAdapter(Baz, (IBar,), IFoo, '')
        bar = Bar()
        adapted = IFoo(bar)
        self.assertTrue(adapted.__class__ is Baz)
        self.assertTrue(adapted.context is bar)

    def test_hit_registered_for_None(self):
        from zope.interface import Interface
        from zope.interface import implementer

        from zope.component import getGlobalSiteManager

        class IFoo(Interface):
            pass

        @implementer(IFoo)
        class Baz:
            def __init__(self, context):
                self.context = context
        getGlobalSiteManager().registerAdapter(Baz, (None,), IFoo, '')
        ctx = object()
        adapted = IFoo(ctx)
        self.assertTrue(adapted.__class__ is Baz)
        self.assertTrue(adapted.context is ctx)
